// If this code works, it was written by Svetlio Kapralov. If not, I don't know who wrote it.
package net.playerfinder.jsf.components.image;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;

import javax.faces.application.ResourceDependencies;
import javax.faces.application.ResourceDependency;
import javax.faces.component.FacesComponent;
import javax.faces.component.UIInput;
import javax.faces.component.behavior.ClientBehaviorHolder;

@FacesComponent(UIImageCropper.COMPONENT_TYPE)
@ResourceDependencies({
		@ResourceDependency(name = "jquery.js", library = "pfjs"),
		@ResourceDependency(name = "pf.no.conflicts.js", library = "pfjs"),
		@ResourceDependency(name = "jquery.color.js", library = "pfjs"),
		@ResourceDependency(name = "jquery.Jcrop.js", library = "pfjs"),
		@ResourceDependency(name = "jquery.Jcrop.css", library = "css") })
public class UIImageCropper extends UIInput implements ClientBehaviorHolder {

	public static final String COMPONENT_TYPE = "net.playerfinder.jsf.components.image.UIImageCropper";
	public static final String COMPONENT_FAMILY = "net.playerfinder.jsf.components";
	public static final String DEFAULT_RENDERER = "net.playerfinder.jsf.components.image.UIImageCropperRenderer";

	private final static String DEFAULT_EVENT = "crop";
	private static final Collection<String> EVENT_NAMES = Collections
			.unmodifiableCollection(Arrays.asList(DEFAULT_EVENT));

	protected enum PropertyKeys {

		imageSrc, imageAltText, aspectRatio, minSize, maxSize, backgroundColor, backgroundOpacity, 
		initialCoords, allowSelect, allowMove, allowResize;

		String toString;

		PropertyKeys(String toString) {
			this.toString = toString;
		}

		PropertyKeys() {
		}

		public String toString() {
			return ((this.toString != null) ? this.toString : super.toString());
		}
	}

	public UIImageCropper() {
		setRendererType(DEFAULT_RENDERER);
	}

	public String getFamily() {
		return COMPONENT_FAMILY;
	}

	@Override
	public Collection<String> getEventNames() {
		return EVENT_NAMES;
	}

	@Override
	public String getDefaultEventName() {
		return DEFAULT_EVENT;
	}

	public String getImageSrc() {
		return (String) getStateHelper().eval(PropertyKeys.imageSrc, null);
	}

	public void setImageSrc(String imageSrc) {
		getStateHelper().put(PropertyKeys.imageSrc, imageSrc);
	}

	public String getImageAltText() {
		return (String) getStateHelper().eval(PropertyKeys.imageAltText, null);
	}

	public void setImageAltText(String imageAltText) {
		getStateHelper().put(PropertyKeys.imageAltText, imageAltText);
	}

	public double getAspectRatio() {
		return (Double) getStateHelper().eval(PropertyKeys.aspectRatio,
				java.lang.Double.MIN_VALUE);
	}

	public void setAspectRatio(double aspectRatio) {
		getStateHelper().put(PropertyKeys.aspectRatio, aspectRatio);
	}

	public String getMinSize() {
		return (String) getStateHelper().eval(PropertyKeys.minSize, null);
	}

	public void setMinSize(String minSize) {
		getStateHelper().put(PropertyKeys.minSize, minSize);
	}

	public String getMaxSize() {
		return (String) getStateHelper().eval(PropertyKeys.maxSize, null);
	}

	public void setMaxSize(String maxSize) {
		getStateHelper().put(PropertyKeys.maxSize, maxSize);
	}

	public String getBackgroundColor() {
		return (String) getStateHelper().eval(PropertyKeys.backgroundColor,
				null);
	}

	public void setBackgroundColor(String backgroundColor) {
		getStateHelper().put(PropertyKeys.backgroundColor, backgroundColor);
	}

	public double getBackgroundOpacity() {
		return (Double) getStateHelper().eval(PropertyKeys.backgroundOpacity,
				0.5);
	}

	public void setBackgroundOpacity(double backgroundOpacity) {
		getStateHelper().put(PropertyKeys.backgroundOpacity, backgroundOpacity);
	}

	public String getInitialCoords() {
		return (String) getStateHelper().eval(PropertyKeys.initialCoords, null);
	}

	public void setInitialCoords(String initialCoords) {
		getStateHelper().put(PropertyKeys.initialCoords, initialCoords);
	}

	public Boolean isAllowSelect() {
		return (Boolean) getStateHelper().eval(PropertyKeys.allowSelect,
				Boolean.TRUE);
	}

	public void setAllowSelect(Boolean allowSelect) {
		getStateHelper().put(PropertyKeys.allowSelect, allowSelect);
	}

	public Boolean isAllowResize() {
		return (Boolean) getStateHelper().eval(PropertyKeys.allowResize,
				Boolean.TRUE);
	}

	public void setAllowResize(Boolean allowResize) {
		getStateHelper().put(PropertyKeys.allowResize, allowResize);
	}

	public Boolean isAllowMove() {
		return (Boolean) getStateHelper().eval(PropertyKeys.allowMove,
				Boolean.TRUE);
	}

	public void setAllowMove(Boolean allowMove) {
		getStateHelper().put(PropertyKeys.allowMove, allowMove);
	}

}